'use strict';
let md5s = null;
const route = require('./router');
const {decrypt} = require('../plug-in/key');
const checkIp = require('../plug-in/checkClientIp');
const serverMd5 = require('../plug-in/serverMd5');
const serverCode = require('../plug-in/serverCode');
module.exports = async (obj,conf) => {
    const { key,secret} = conf;
    const code = serverCode(key, secret);
    let rest = { code: 0, msg: 'An unknown error！' };
    if (code == null) {
        // 验证serverCode
        rest.msg = 'The serverCode error！';
        return rest;
    }
    if(conf.open==false&&obj.url=='/edit'){
        if(!obj.data['open']) {
            rest.msg = 'The cluster has been closed！';
            return rest;
        } 
    }
    const arr = route[obj.url];
    if(!arr) {
        rest.msg = 'The route error！';
        return rest;
    }
    if (arr.includes('ip')) {
        // 验证ip白名单
        if (!checkIp(obj.ip)) {
            // 没通过白名单
            rest.msg = "IP does not exist!";
            return rest;
        }
    }
    if (arr.includes('md5')) {
        // 验证请求md5
        let data = obj.data;
        const md5 = obj.data.md5;
        delete data.md5;
        if (md5s && md5s == md5) {
            // 验证md5值实效性,具有防抖效果
            rest.msg = 'The md5 error！';
            return rest;
        }
        md5s = md5;
        const svmd5 = serverMd5(code, data);
        if (md5 !== svmd5) {
            // 验证fk2传来的md5值实效性
            rest.msg = 'The md5 error！';
            return rest;
        }
    }
    if (arr.includes('token')) {
        // 验证请求来自客户端的token 
        if(obj.data.token&&obj.data.id){
            // 开始验证
            try {
                const expired = 10*60*1000;  //token过期时间
                const time = new Date().getTime();
                const dres = JSON.parse(decrypt(obj.data.token));
                const times = time-dres.time;
                if(dres.code!==code||dres.ip!==obj.ip||times>expired){
                    rest.msg = 'Token expiration！';
                    return rest;
                }
            } catch (error) {
                rest.msg = 'Token validation error！';
                return rest;
            }
        }else{
            rest.msg = 'The token error！';
            return rest;
        }
    }
    try {
        const { main } = require('./api' + obj.url);
        return await main(obj,conf);
    } catch (e) {
        return rest;
    }
};